from bunkatopics import BunkaTopics
import pandas as pd

data = pd.read_csv("../data/imdb.csv", index_col=[0])
data = data.sample(500, random_state=42)

# Instantiate the model, extract ther terms and Embed the documents

if __name__ == "__main__":

    model = BunkaTopics(
        data,  # dataFrame
        text_var="description",  # Text Columns
        index_var="imdb",  # Index Column (Mandatory)
        extract_terms=True,  # extract Terms ?
        terms_embeddings=True,  # extract terms Embeddings?
        docs_embeddings=True,  # extract Docs Embeddings?
        embeddings_model="distiluse-base-multilingual-cased-v1",  # Chose an embeddings Model
        multiprocessing=True,  # Multiprocessing of Embeddings
        language="en",  # Chose between English "en" and French "fr"
        sample_size_terms=len(data),
        terms_limit=10000,  # Top Terms to Output
        terms_ents=True,  # Extract entities
        terms_ngrams=(1, 2),  # Chose Ngrams to extract
        terms_ncs=True,  # Extract Noun Chunks
        terms_include_pos=["NOUN", "PROPN", "ADJ"],  # Include Part-of-Speech
        terms_include_types=["PERSON", "ORG"],
    )  # Include Entity Types

    # Extract the topics

    topics = model.get_clusters(
        topic_number=15,  # Number of Topics
        top_terms_included=1000,  # Compute the specific terms from the top n terms
        top_terms=5,  # Most specific Terms to describe the topics
        term_type="lemma",  # Use "lemma" of "text"
        ngrams=[1, 2],
    )  # N-grams for Topic Representation

    # Visualize the clusters. It is adviced to choose less that 5 terms - top_terms = 5 - to avoid overchanging the Figure

    fig = model.visualize_clusters(search=None, width=1000, height=1000)
    fig.show()



import spacy
from multiprocessing import Pool
from tqdm import tqdm
import pandas as pd

nlp = spacy.load("fr_core_news_lg")


def get_sent(text):
    doc = nlp(text)
    res = [sentence.text for sentence in doc.sents]
    return res


def sent_extract(docs):
    res = list(tqdm(map(get_sent, docs), total=len(docs)))

    return res


def sent_multiprocessing(docs):
    with Pool(8) as p:
        res = list(tqdm(p.imap(get_sent, docs), total=len(docs)))

    return res



import plotly.graph_objs as go


def get_density_plot(x, y, width, height):

    fig_density = go.Figure(go.Histogram2dContour(x=x, y=y, colorscale="delta"))

    fig_density.update_traces(contours_coloring="fill", contours_showlabels=False)

    fig_density.update_layout(
        font_size=25,
        width=width,
        height=height,
        margin=dict(t=200),
        title=dict(font=dict(size=width / 40)),
    )

    fig_density.update_layout(showlegend=False)

    return fig_density



import pandas as pd
from tqdm import tqdm
import pickle
from multiprocessing import Pool
import multiprocessing
import textacy
import textacy.preprocessing
import textacy.representations
import textacy.tm
from functools import partial
import warnings

# import en_core_web_sm
# import fr_core_news_lg

tqdm.pandas()
# lang = en_core_web_sm.load()

warnings.simplefilter(action="ignore", category=FutureWarning)

preproc = textacy.preprocessing.make_pipeline(
    textacy.preprocessing.normalize.unicode,
    textacy.preprocessing.normalize.bullet_points,
    textacy.preprocessing.normalize.quotation_marks,
    textacy.preprocessing.normalize.whitespace,
    textacy.preprocessing.normalize.hyphenated_words,
    textacy.preprocessing.remove.brackets,
    textacy.preprocessing.replace.currency_symbols,
    textacy.preprocessing.remove.html_tags,
)


def from_dict_to_frame(indexed_dict):
    data = {k: [v] for k, v in indexed_dict.items()}
    df = pd.DataFrame.from_dict(data).T
    df.columns = ["text"]
    df = df.explode("text")
    return df


def extract_terms_df(
    data,
    text_var,
    index_var,
    ngs=True,
    ents=True,
    ncs=False,
    multiprocess=True,
    sample_size=100000,
    drop_emoji=True,
    ngrams=(2, 2),
    remove_punctuation=True,
    include_pos=["NOUN", "PROPN", "ADJ"],
    include_types=["PERSON", "ORG"],
    language="en",
):

    if language == "zh":
        # nlp = zh_core_web_sm.load()
        lang = textacy.load_spacy_lang("zh_core_web_sm", disable=())
    elif language == "en":
        # nlp = en_core_web_sm.load()
        lang = textacy.load_spacy_lang("en_core_web_sm", disable=())
    elif language == "fr":
        # nlp = fr_core_news_lg.load()
        lang = textacy.load_spacy_lang("fr_core_news_lg", disable=())
    else:
        pass

    def extract_terms(
        tuple,  # (index, text)
        ngs=True,
        ents=True,
        ncs=False,
        ngrams=(2, 2),
        drop_emoji=True,
        remove_punctuation=False,
        include_pos=["NOUN", "PROPN", "ADJ"],
        include_types=["PERSON", "ORG"],
    ):

        index = tuple[0]
        text = tuple[1]

        prepro_text = preproc(str(text))
        if drop_emoji == True:
            prepro_text = textacy.preprocessing.replace.emojis(prepro_text, repl="")

        if remove_punctuation == True:
            prepro_text = textacy.preprocessing.remove.punctuation(prepro_text)

        doc = textacy.make_spacy_doc(prepro_text, lang=lang)

        terms = []

        if ngs:
            ngrams_terms = list(
                textacy.extract.terms(
                    doc,
                    ngs=partial(
                        textacy.extract.ngrams,
                        n=ngrams,
                        filter_punct=True,
                        filter_stops=True,
                        include_pos=include_pos,
                    ),
                    dedupe=False,
                )
            )

            terms.append(ngrams_terms)

        if ents:
            ents_terms = list(
                textacy.extract.terms(
                    doc,
                    ents=partial(textacy.extract.entities, include_types=include_types),
                    dedupe=False,
                )
            )
            terms.append(ents_terms)

        if ncs:
            ncs_terms = list(
                textacy.extract.terms(
                    doc,
                    ncs=partial(textacy.extract.noun_chunks, drop_determiners=True),
                    dedupe=False,
                )
            )

            noun_chunks = [x for x in ncs_terms if len(x) >= 3]
            terms.append(noun_chunks)

        final = [item for sublist in terms for item in sublist]
        final = list(set(final))

        df = [
            (term.text, term.lemma_.lower(), term.label_, term.__len__())
            for term in final
        ]
        df = pd.DataFrame(df, columns=["text", "lemma", "ent", "ngrams"])
        df["text_index"] = index

        return df

    """
    This function extracts terms from a column in a DataFrame. It can extract in a multiprocessing way
    It outputs a dataframe with the list of terms and a table with the indexed terms
    """

    data = data[data[text_var].notna()]
    data = data.sample(min(sample_size, len(data)))

    sentences = data[text_var].to_list()
    indexes = data[index_var].to_list()
    inputs = [(x, y) for x, y in zip(indexes, sentences)]

    """if multiprocess:
        with Pool(multiprocessing.cpu_count() - 2) as p:
            res = list(
                tqdm(
                    p.imap(
                        partial(
                            extract_terms,
                            ngs=ngs,
                            ents=ents,
                            ncs=ncs,
                            drop_emoji=drop_emoji,
                            remove_punctuation=remove_punctuation,
                            ngrams=ngrams,
                            include_pos=include_pos,
                            include_types=include_types,
                        ),
                        inputs,
                    ),
                    total=len(inputs),
                )
            )

        final_res = pd.concat([x for x in res])"""

    # else:
    res = list(
        tqdm(
            map(
                partial(
                    extract_terms,
                    ngs=ngs,
                    ents=ents,
                    ncs=ncs,
                    drop_emoji=drop_emoji,
                    remove_punctuation=remove_punctuation,
                    ngrams=ngrams,
                    include_pos=include_pos,
                    include_types=include_types,
                ),
                inputs,
            ),
            total=len(inputs),
        )
    )

    final_res = pd.concat([x for x in res])

    terms = (
        final_res.groupby(["text", "lemma", "ent", "ngrams"])
        .agg(count_terms=("text_index", "count"))
        .reset_index()
    )

    # duplicates to get rid of
    terms = terms.sort_values(["text", "ent"]).reset_index()
    terms = terms.drop_duplicates(["text"], keep="first")
    terms = terms.sort_values("count_terms", ascending=False)
    terms = terms.set_index("text")

    terms_indexed = final_res[["text", "text_index"]].drop_duplicates()
    terms_indexed = terms_indexed.rename(columns={"text_index": index_var})

    terms_indexed = terms_indexed.groupby(index_var)["text"].apply(list)

    return terms, terms_indexed



import pandas as pd
from sentence_transformers import SentenceTransformer
from sklearn.feature_extraction.text import TfidfVectorizer
from multiprocessing import Pool
from tqdm import tqdm
import umap

model = SentenceTransformer("distiluse-base-multilingual-cased-v1")


def multi_embed(text):
    emb = model.encode(text, show_progress_bar=False)
    return emb


def get_embeddings(data, index_var, text_var, multiprocessing=True, reduction=5):
    # Embed documents

    data = data[data[text_var].notna()]
    docs = data[text_var].to_list()
    indexes = data[index_var].to_list()

    if multiprocessing:
        with Pool(8) as p:
            embeddings = list(tqdm(p.imap(multi_embed, docs), total=len(docs)))
        df_embeddings = pd.DataFrame(embeddings, index=indexes)
    else:
        embeddings = model.encode(docs, show_progress_bar=True)
        df_embeddings = pd.DataFrame(embeddings)
        df_embeddings.index = indexes

    if reduction is not None:
        red = umap.UMAP(n_components=reduction, verbose=True, random_state=42)
        red.fit(embeddings)
        embeddings = red.transform(embeddings)
        df_embeddings = pd.DataFrame(embeddings)
        df_embeddings.index = indexes

    return df_embeddings



import pandas as pd
from .multiprocess_embeddings import get_embeddings
from .extract_terms import extract_terms_df
from .sent_multiprocessing import sent_multiprocessing, sent_extract
from .indexer import indexer
import logging

logging.basicConfig(format="%(asctime)s - %(message)s", level=logging.INFO)


class BasicSemantics:
    """This Class carries out the basic operations that all following models will use
    - terms extraction
    - docs embeddings
    - terms embeddings
    - Sentence Extraction
    - terms indexation

    This class will be used as Parent of other specialized class

    """

    def __init__(
        self,
        data,
        text_var,
        index_var,
        terms_path=None,
        terms_embeddings_path=None,
        docs_embeddings_path=None,
    ):
        self.data = data[data[text_var].notna()].reset_index(drop=True)
        self.text_var = text_var
        self.index_var = index_var

        self.terms_path = terms_path
        self.terms_embeddings_path = terms_embeddings_path
        self.docs_embeddings_path = docs_embeddings_path

        # Load existing dataset if they exist
        """if terms_path is not None:
            self.terms = pd.read_csv(terms_path, index_col=[0])
            self.index_terms(projection=False, db_path=".")"""

        if terms_embeddings_path is not None:
            self.terms_embeddings = pd.read_csv(terms_embeddings_path, index_col=[0])

        if docs_embeddings_path is not None:
            self.docs_embeddings = pd.read_csv(docs_embeddings_path, index_col=[0])

    def fit(
        self,
        extract_terms=True,
        terms_embeddings=True,
        docs_embeddings=False,
        sample_size_terms=500,
        terms_limit=500,
        terms_ents=True,
        terms_ngrams=(1, 2),
        terms_ncs=True,
        terms_include_pos=["NOUN", "PROPN", "ADJ"],
        terms_include_types=["PERSON", "ORG"],
        embeddings_model="distiluse-base-multilingual-cased-v1",
        multiprocessing=True,
        reduction=5,
        language="en",
    ):

        if self.terms_embeddings_path is not None:
            terms_embeddings = False
        if self.docs_embeddings_path is not None:
            docs_embeddings = False
        if self.terms_path is not None:
            extract_terms = False

        if extract_terms:
            logging.info("Extracting Terms...")
            self.terms = self.extract_terms(
                sample_size=sample_size_terms,
                limit=terms_limit,
                ents=terms_ents,
                ncs=terms_ncs,
                ngrams=terms_ngrams,
                include_pos=terms_include_pos,
                include_types=terms_include_types,
                language=language,
                multiprocessing=multiprocessing,
            )

        if docs_embeddings:
            logging.info("Extracting Docs Embeddings...")
            self.docs_embeddings = self.extract_docs_embeddings(
                multiprocessing=multiprocessing, reduction=reduction
            )
        if terms_embeddings:
            logging.info("Extracting Terms Embeddings...")
            self.terms_embeddings = self.extract_terms_embeddings(
                multiprocessing=multiprocessing, reduction=reduction
            )

        self.docs_embedding_model = embeddings_model

        self.data = self.data.set_index(self.index_var)

    def extract_terms(
        self,
        sample_size,
        limit,
        ents=True,
        ncs=True,
        ngrams=(1, 2),
        include_pos=["NOUN", "PROPN", "ADJ"],
        include_types=["PERSON", "ORG"],
        language="en",
        multiprocessing=True,
    ):
        self.terms, terms_indexed = extract_terms_df(
            self.data,
            text_var=self.text_var,
            index_var=self.index_var,
            ngs=True,
            ents=ents,
            ncs=ncs,
            multiprocess=multiprocessing,
            sample_size=sample_size,
            drop_emoji=True,
            ngrams=ngrams,
            remove_punctuation=False,
            include_pos=include_pos,
            include_types=include_types,
            language=language,
        )

        self.terms = self.terms.sort_values("count_terms", ascending=False)
        self.terms = self.terms.head(limit)

        self.df_terms_indexed = terms_indexed.copy()

        return self.terms

    def extract_docs_embeddings(self, multiprocessing=True, reduction=5):
        # Extract Embeddings
        self.df_docs_embeddings = get_embeddings(
            self.data,
            index_var=self.index_var,
            text_var=self.text_var,
            multiprocessing=multiprocessing,
            reduction=reduction,
        )

        return self.df_docs_embeddings

    def extract_terms_embeddings(self, multiprocessing=True, reduction=5):
        # Extract Embeddings
        self.df_terms_embeddings = get_embeddings(
            self.terms.reset_index(),
            index_var="index",
            text_var="text",
            multiprocessing=multiprocessing,
            reduction=reduction,
        )

        self.terms = self.terms.drop("index", axis=1)

        self.df_terms_embeddings.index = self.terms.index

        return self.df_terms_embeddings

    def extract_sentences(self, multiprocess=True):
        docs = self.data[self.text_var]
        if multiprocess:
            res = sent_multiprocessing(docs)
        else:
            res = sent_extract(docs)

        indexes = self.data[self.index_var].to_list()
        final_dict = {"sentences": res, "doc_index": indexes}

        self.df_sent = pd.DataFrame(final_dict)

        return self.df_sent

    def index_terms(self, list_to_index):

        docs = self.data[self.text_var].to_list()
        res = indexer(list_to_index, docs)

        res = res.groupby("docs")["indexed_term"].apply(list).reset_index()
        res.columns = [self.text_var, "text"]
        res = pd.merge(
            res, self.data[[self.text_var, self.index_var]], on=self.text_var
        )

        final_res = res.copy()
        self.df_terms_indexed = final_res.drop(self.text_var, axis=1)
        self.df_terms_indexed = self.df_terms_indexed.set_index(self.index_var)

        return res



def wrap_by_word(string, n_words):
    """returns a string where \\n is inserted between every n words"""
    try:
        a = string.split()
        ret = ""
        for i in range(0, len(a), n_words):
            ret += " ".join(a[i : i + n_words]) + "<br>"
    except:
        pass

    return ret



from distutils.core import setup

setup(
    name="bunkatopics",
    packages=["bunkatopics"],
    version="0.15",
    license="MIT",
    description="Advanced Topic Modeling Using Transformers",
    author="Charles de Dampierre",
    author_email="charles.de-dampierre@hec.edu",
    url="https://github.com/charlesdedampierre/BunkaTopics",
    download_url="https://github.com/charlesdedampierre/BunkaTopics/archive/v_01.tar.gz",
    keywords=[
        "Topic Modeling",
        "NLP",
        "Search",
    ],
    install_requires=[
        "pandas==1.4.1",
        "scikit_learn==1.1.1",
        "sentence_transformers==2.2.0",
        "textacy==0.12.0",
        "tqdm==4.63.0",
        "umap-learn",
        # "spacy== 3.2.3",
        # "en_core_web_sm",
        # "fr_core_news_lg",
        "plotly"
        # "en_core_web_sm @ https://github.com/explosion/spacy-models/releases/download/en_core_web_sm-3.0.0/en_core_web_sm-3.0.0.tar.gz",
    ],
    # dependency_links=[
    # "https://github.com/explosion/spacy-models/releases/download/en_core_web_sm-3.0.0/en_core_web_sm-3.0.0.tar.gz#egg=en_core_web_sm"
    # ],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
    ],
)



import pandas as pd
from tqdm import tqdm


def indexer(list_to_index, docs):

    final = pd.DataFrame()
    for term in tqdm(list_to_index, total=len(list_to_index)):
        new = pd.DataFrame(docs, columns=["docs"])
        res = new[new["docs"].str.contains(term, case=False)]
        res["indexed_term"] = term
        final = final.append(res)

    final = final.drop_duplicates().reset_index(drop=True)

    return final
